// 極楽画像計測（Gokuraku_measure）用データファイル結合プログラム(Gokuraku_merge)
// Created by Tatsuya Shirai
// National Institute of Technology, Suzuka college
// Mechanical Department
//
// Ver. 1.0 : 2022.06.08
// Ver. 2.0 : テキストファイル名の代わりに画像ファイル名を記録する（パス無し）
// Ver. 2.2 : データを読み込まずにエクスポートフォルダーを設定しよとするとバグる．Ｉコマンドをファイル読み込みではなくフォルダーからの読み込みに変更
//            ソーティング機能を実装，dataが単方向リストなのを配列に変更する．

Data       data;
ImportFile im_file;
ExportFile ex_file;
Sorting    sorting;
MenuScreen menuScreen;

final int Scr_w = 800;
final int Scr_h = 460;

enum Menu_status {main, sorting};
Menu_status menu_status;

void setup() {
  data         = new Data();
  sorting      = new Sorting();
  im_file      = new ImportFile();
  ex_file      = new ExportFile();
  shortMessage = new ShortMessage();
  menu_status  = Menu_status.main;
  ci = new CurrentImage(); // 互換性

  //サイズ変更を許可する（実際に変更することはない）
  surface.setResizable(true);
  surface.setSize(Scr_w, Scr_h);

  // 日本語フォントを使用する
  PFont font = createFont(JapaneseFont, 50);
  textFont(font);

  drop_init();
}

void draw(){
  menuScreen = new MenuScreen(16, 40, 150, 40);

  // 画面消去
  fill(72, 72, 72);
  rect(0, 0, Scr_w - 1, Scr_h - 1);

  // タイトル表示
  menuScreen.display_title("Gokuraku merge", 80, 255, 80);

  // メニュー表示
  textSize(24);
  fill(255, 80, 80);
  if (menu_status == Menu_status.main) {
    menuScreen.display_menu("[I]",           "import measured data files from folder");
    menuScreen.display_menu("(Right clisk)", "import a mesured data file");
    menuScreen.display_menu("[C]",           "clear data");
    menuScreen.display_menu("[S]",           "open sub-menu (sorting)");
    menuScreen.display_menu("[F]",           "select export folder");
    menuScreen.display_menu("[E]",           "export merged data fils");
    menuScreen.display_menu("[ESC]",         "quit");

    menuScreen.y += 16;
    String str = "Folder: ";
    if (ex_file.export_folder == "") str += "not selected";             
    else                             str += ex_file.export_folder;
    menuScreen.display_note(str);
    str = "Imported ";
    if (data.num > 0)                str += data.num + " files";
    else                             str += "no files";
    menuScreen.display_note(str);
  } else if (menu_status == Menu_status.sorting) {
    menuScreen.dx = 64;
    menuScreen.display_menu("[N]", "Sort by filename");
    menuScreen.display_menu("[T]", "Sort by time");
    menuScreen.display_menu("[Q]", "quit sub-menu"); 

    String str;
    str  = "Order condition (by Filename) : " + (sorting.checkByFilename() ? "OK" : "NG");
    str += "/line: " + (sorting.line < 0 ? "no data" : sorting.line + 1);
    menuScreen.display_note(str);
    str  = "Order condition (by Time    ) : " + (sorting.checkByTime()     ? "OK" : "NG");
    str += "/line: " + (sorting.line < 0 ? "no data" : sorting.line + 1);
    menuScreen.display_note(str);
  }
  menuScreen.y += menuScreen.dy;
  shortMessage.display_short_message(menuScreen.x, menuScreen.y);
}

void mousePressed()
{
  if (mouseButton == RIGHT) open_fileselector();
}

void keyPressed()
{
  File fp;
  if (menu_status == Menu_status.main) {
    switch (keyCode) {
      case 'I' :
        fp = new File(im_file.import_folder);
        selectFolder("Select a import folder:", "imfolderSelected", fp);
        break;
      case 'F' :
        // エクスポートするフォルダーの選択
        fp = new File(ex_file.export_folder);
        selectFolder("Select a export folder:", "exfolderSelected", fp);
        break;
      case 'C' : 
        // 全データ消去
        data.allclear();
        shortMessage.set_message("Clear all data");
        break;
      case 'S' :
        // ソーティングのサブメニューに移動
        sorting.refreshCheck();
        menu_status = Menu_status.sorting;
        break;
      case 'E' : 
        // 全データのファイルへの出力
        ex_file.export_alldata(data);
        break;
    }
  } else if (menu_status == Menu_status.sorting) {
    switch (keyCode) {
      case 'N' :
        if (data.num <= 0) break;
        sorting.sortByFilename();
        shortMessage.set_message("Sort by Filename: " + data.num + " data");
        break;
      case 'T' :
        if (data.num <= 0) break;
        sorting.sortByTime();
        shortMessage.set_message("Sort by Time: " + data.num + " data");
        break;
      case 'Q' :
        menu_status = Menu_status.main;
        break;
    }
    key = 0;
  }
}

// ファイル選択ダイアログボックスの呼び出し
void open_fileselector()
{
  selectInput("Select a measured data file:", "fileSelected");
}
// ファイル選択後の処理
void fileSelected(File selection) 
{
  if (selection == null) return;
  im_file.read_and_store(selection.getAbsolutePath(), data);
  im_file.import_folder = selection.getAbsolutePath();
}
// インポートフォルダー選択後の処理
void imfolderSelected(File selection) {
  if (selection == null) return;
  im_file.import_folder = selection.getAbsolutePath();
  im_file.open_files(selection.listFiles());
}

// エクスポートするフォルダーの選択
void exfolderSelected(File selection) {
  if (selection == null) return;  // キャンセル時
  ex_file.set_export_folder(selection.getAbsolutePath());
  shortMessage.set_message("User select folder = " + ex_file.export_folder);
}

//// ドラッグ＆ドロップ関係
// 上記URL参照
// ドロップされたファイルの処理
void fileSelected(List<File> fs) {
  File [] fileList = {};
  for(File f:fs){
    // ドラッグ＆ドロップされたリストの中にフォルダーが含まれていた場合は，その層までは自動的に読み込む
    if (f.isDirectory()) {
      for (File sf:f.listFiles()) {
        if (sf.isFile()) fileList = (File [])append(fileList, sf);  // ListからArrayへ変更
      }
    }
    fileList = (File [])append(fileList, f);  // ListからArrayへ変更
  }
  if (fileList.length == 0) return;
  im_file.open_files(fileList);
}

// ライブラリー互換性のための宣言（mergeでは使用しない機能）
final int Define_stick[][] = {};
class Preview {
  boolean enable_scale = false;
  Preview() {
  }
}
class CurrentImage {
  boolean  enable_scale;
  Plot     plotMark;
  Preview  preview;
  CurrentImage() {
    enable_scale = false;
    plotMark     = new Plot();
    preview      = new Preview();
  }
}
CurrentImage ci;
final int    Plot_size        = 5;                  // ターゲットマーカーのサイズ
final int    Plot_color[][]   = {{255,  80,  80},   // ターゲットマーカーの色 : 赤，白，黒 
                                 {255, 255, 255},
                                 {0, 0, 0}};  
